/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: svg.c,v 1.4.2.1 2002/11/14 13:25:24 makeinu Exp $
 */

#include "svg.h"

#ifdef ENABLE_SVG

#ifdef HAVE_GDK_PIXBUF

#include <stdio.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <librsvg/rsvg.h>
#include "fileutil.h"

GimvImage *
svg_load_image (ImageLoader *loader, gpointer data)
{
   const gchar *filename;
   gfloat w_scale, h_scale;
   GimvImage *image = NULL;
   GdkPixbuf *pixbuf;

   g_return_val_if_fail (loader, NULL);

   filename = image_loader_get_path (loader);
   g_return_val_if_fail (filename, NULL);

   image_loader_get_scale (loader, &w_scale, &h_scale);
   g_return_val_if_fail (w_scale > 0.0 && h_scale > 0.0, NULL);

#ifdef USE_GTK2
   {
      GError *error;
      pixbuf = rsvg_pixbuf_from_file_at_zoom (filename,
                                              w_scale,
                                              h_scale,
                                              &error);
   }
#else /* USE_GTK2 */
   {
      FILE *file;

      file = fopen (filename, "r");
      if (!file) return NULL;

      pixbuf = rsvg_render_file (file, MIN (w_scale, h_scale));

      fclose (file);
   }
#endif /* USE_GTK2 */

   if (pixbuf) {
      image = gimv_image_new ();
      image->image = pixbuf;
      image->flags |= GIMV_IMAGE_VECTOR_FLAGS;
      return image;
   } else {
      return NULL;
   }
}

#else /* HAVE_GDK_PIXBUF */

GimvImage *
svg_load_image (ImageLoader *loader, gpointer data)
{
   return NULL;
}

#endif /* HAVE_GDK_PIXBUF */
#endif /* ENABLE_SVG */
