/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2002 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: prefs_ui_table.c,v 1.8 2002/10/31 12:21:10 makeinu Exp $
 */

#include "prefs_ui_table.h"

#include "gtk2-compat.h"
#include "gtkutils.h"
#include "intl.h"
#include "prefs.h"
#include "prefs_win.h"

extern Config   *config_changed;
extern Config   *config_prechanged;


static void
cb_rotate_min_colnum_value (GtkWidget *widget, GtkWidget *spinner)
{
   gint colmin;

   colmin = GTK_ADJUSTMENT(widget)->value;

   if (colmin > config_changed->thumbtable_colnum_max) {
      config_changed->thumbtable_colnum_min = config_changed->thumbtable_colnum_max; 
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinner),
                                 (gfloat) config_changed->thumbtable_colnum_min);
   } else {
      config_changed->thumbtable_colnum_min = colmin; 
   }
}


static void
cb_rotate_max_colnum_value (GtkWidget *widget, GtkWidget *spinner)
{
   gint colmax;

   colmax = GTK_ADJUSTMENT(widget)->value;

   if (colmax < config_changed->thumbtable_colnum_min) {
      config_changed->thumbtable_colnum_max = config_changed->thumbtable_colnum_min; 
      gtk_spin_button_set_value (GTK_SPIN_BUTTON(spinner),
                                 (gfloat) config_changed->thumbtable_colnum_max);
   } else {
      config_changed->thumbtable_colnum_max = colmax; 
   }
}


/*******************************************************************************
 *  prefs_thumbtable_page:
 *     @ Create Thumbnail Table preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_thumbtable_page (void)
{
   GtkWidget *main_vbox;
   GtkWidget *frame, *frame_vbox, *table, *vbox, *hbox;
   GtkWidget *label;
   GtkAdjustment *adj;
   GtkWidget *toggle, *spinner;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /********************************************** 
    * Thumbnail Table Frame
    **********************************************/
   prefs_ui_create_frame (_("Thumbnail Table"),
                          frame, frame_vbox, main_vbox, FALSE);

   hbox = gtk_hbox_new (FALSE, 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, FALSE, FALSE, 0);

   table = gtk_table_new (2, 4, FALSE);
   gtk_table_set_row_spacings (GTK_TABLE (table), 5);
   gtk_table_set_col_spacings (GTK_TABLE (table), 5);
   gtk_container_set_border_width(GTK_CONTAINER(table), 5);
   gtk_box_pack_start (GTK_BOX (hbox), table, FALSE, FALSE, 0);

   /* Minimum column number spinner */
   label = gtk_label_new (_("Minimum Columns"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                     GTK_EXPAND, GTK_FILL, 0, 0);

   adj = (GtkAdjustment *) gtk_adjustment_new (conf.thumbtable_colnum_min,
                                               1.0, 256.0, 1.0, 5.0, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (cb_rotate_min_colnum_value), spinner);
   gtk_table_attach (GTK_TABLE (table), spinner, 1, 2, 0, 1,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Maximum column number spinner */
   label = gtk_label_new (_("Maximum Columns"));
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
                     GTK_EXPAND, GTK_FILL, 0, 0);

   adj = (GtkAdjustment *) gtk_adjustment_new (conf.thumbtable_colnum_max,
                                               1.0, 256.0, 1.0, 5.0, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (cb_rotate_max_colnum_value), spinner);
   gtk_table_attach (GTK_TABLE (table), spinner, 3, 4, 0, 1,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Row Spacing spinner */
   label = gtk_label_new (_("Row Spacing"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                     GTK_EXPAND, GTK_FILL, 0, 0);

   adj = (GtkAdjustment *) gtk_adjustment_new (conf.thumbtable_row_space,
                                               0.0, 255.0, 1.0, 5.0, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (gtkutil_get_data_from_adjustment_by_int_cb),
                       &config_changed->thumbtable_row_space);
   gtk_table_attach (GTK_TABLE (table), spinner, 1, 2, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Column Spacing spinner */
   label = gtk_label_new (_("Column Spacing"));
   gtk_table_attach (GTK_TABLE (table), label, 2, 3, 1, 2,
                     GTK_EXPAND, GTK_FILL, 0, 0);

   adj = (GtkAdjustment *) gtk_adjustment_new (conf.thumbtable_col_space,
                                               0.0, 255.0, 1.0, 5.0, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (gtkutil_get_data_from_adjustment_by_int_cb),
                       &config_changed->thumbtable_col_space);
   gtk_table_attach (GTK_TABLE (table), spinner, 3, 4, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);

   /* Move to new tab automatically or not */
   toggle = gtkutil_create_check_button (_("Centering"),
                                         conf.thumbtable_centering,
                                         gtkutil_get_data_from_toggle_cb,
                                         &config_changed->thumbtable_centering);
   gtk_box_pack_start (GTK_BOX (frame_vbox), toggle, FALSE, FALSE, 5);

   /**********************************************
    * Thumbnail Button Frame
    **********************************************/
   prefs_ui_create_frame (_("Thumbnail Button"),
                          frame, vbox, main_vbox, FALSE);

   hbox = gtk_hbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

   /* Minimum border width spinner */
   label = gtk_label_new (_("Minimum Border Width"));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);

   adj = (GtkAdjustment *) gtk_adjustment_new (conf.thumbbutton_border_width,
                                               0.0, 32.0, 1.0, 5.0, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (gtkutil_get_data_from_adjustment_by_int_cb),
                       &config_changed->thumbbutton_border_width);
   gtk_box_pack_start(GTK_BOX(hbox), spinner, FALSE, TRUE, 0);

   gtk_widget_show_all (main_vbox);

   return main_vbox;
}
