/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: thumbview_table2.c,v 1.4.2.1 2002/11/02 11:51:51 makeinu Exp $
 */

#include <string.h>

#include "gimageview.h"

#include "charset.h"
#include "dnd.h"
#include "gtkutils.h"
#include "prefs.h"
#include "thumbnail.h"
#include "thumbnail_view.h"
#include "thumbview_table2.h"
#include "thumbnail_window.h"
#include "zalbum.h"

ThumbViewDispMode thumbtable2_modes[] =
{
   {N_("Thumbnail2"),
    thumbtable2_create,
    thumbtable2_append_thumb_frames,
    thumbtable2_add_thumbnail,
    thumbtable2_refresh_thumbnail,
    thumbtable2_redraw,
    NULL,
    thumbtable2_adjust,
    thumbtable2_remove_thumbview_data,
    NULL,
    thumbtable2_set_selection,
    thumbtable2_thumbnail_is_in_viewport},

   {N_("List2 (Icon)"),
    thumbtable2_create,
    thumbtable2_append_thumb_frames,
    thumbtable2_add_thumbnail,
    thumbtable2_refresh_thumbnail,
    thumbtable2_redraw,
    NULL,
    thumbtable2_adjust,
    thumbtable2_remove_thumbview_data,
    NULL,
    thumbtable2_set_selection,
    thumbtable2_thumbnail_is_in_viewport},

   {N_("List2 (Thumbnail)"),
    thumbtable2_create,
    thumbtable2_append_thumb_frames,
    thumbtable2_add_thumbnail,
    thumbtable2_refresh_thumbnail,
    thumbtable2_redraw,
    NULL,
    thumbtable2_adjust,
    thumbtable2_remove_thumbview_data,
    NULL,
    thumbtable2_set_selection,
    thumbtable2_thumbnail_is_in_viewport},
};
gint thumbtable2_modes_num = sizeof (thumbtable2_modes) / sizeof (ThumbViewDispMode);


typedef struct ThumbViewData_Tag
{
   GtkWidget *album;
} ThumbViewData;


/* defined in thumbview_list.c */
#include "thumbview_list.h"
extern void   listview_create_title_idx_list  (void);
extern gchar *listview_create_label_str       (Thumbnail *thumb);


/******************************************************************************
 *
 *   Callback functions.
 *
 ******************************************************************************/
static gboolean
cb_album_button_press (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
   ThumbView *tv = data;
   gint row;
   Thumbnail *thumb;

   g_return_val_if_fail (tv, TRUE);

   row = zlist_cell_index_from_xy (ZLIST (widget), event->x, event->y);
   if (row < 0) {
      gtk_drag_source_unset (widget);
      gtk_object_set_data (GTK_OBJECT (widget), "drag-unset", GINT_TO_POINTER (1));
      thumbwin_notebook_drag_src_unset (tv->thumb_window);   /* FIXME!! */
      return TRUE;
   }

   thumb = zalbum_get_cell_data (ZALBUM (widget), row);
   if (!thumb) return TRUE;

   thumbview_thumb_button_press_cb (widget, event, thumb);

   return TRUE;
}


static gboolean
cb_album_button_release (GtkWidget *widget, GdkEventButton *event, gpointer data)
{
   ThumbView *tv = data;
   Thumbnail *thumb;
   gint row;
   gpointer dnd_unset;

   g_return_val_if_fail (tv, TRUE);

   dnd_unset = gtk_object_get_data (GTK_OBJECT (widget), "drag-unset");
   if (dnd_unset)
      dnd_src_set  (widget, dnd_types_uri, dnd_types_uri_num);
   dnd_unset = FALSE;

   row = zlist_cell_index_from_xy (ZLIST (widget), event->x, event->y);
   if (row < 0) {
      thumbwin_notebook_drag_src_reset (tv->thumb_window);   /* FIXME!! */
      return TRUE;
   }

   thumb = zalbum_get_cell_data (ZALBUM (widget), row);
   if (!thumb) return TRUE;

   thumbview_thumb_button_release_cb (widget, event, thumb);

   return TRUE;
}


static void
cb_select_cell (ZAlbum *album, gint idx, ThumbView *tv)
{
   Thumbnail *thumb;

   g_return_if_fail (IS_ZALBUM (album));
   g_return_if_fail (tv);

   thumb = zalbum_get_cell_data (ZALBUM (album), idx);
   g_return_if_fail (thumb);

   thumb->selected = TRUE;
}

static void
cb_unselect_cell (ZAlbum *album, gint idx, ThumbView *tv)
{
   Thumbnail *thumb;

   g_return_if_fail (IS_ZALBUM (album));
   g_return_if_fail (tv);

   thumb = zalbum_get_cell_data (ZALBUM (album), idx);
   g_return_if_fail (thumb);

   thumb->selected = FALSE;
}


static gboolean
cb_album_key_press (GtkWidget *widget,
                    GdkEventKey *event,
                    ThumbView *tv)
{
   Thumbnail *thumb = NULL;
   guint keyval, popup_key = 0;
   GdkModifierType modval, popup_mod = 0;

   g_return_val_if_fail (tv, FALSE);

   keyval = event->keyval;
   modval = event->state;

   if (akey.common_popup_menu || *akey.common_popup_menu)
      gtk_accelerator_parse (akey.common_popup_menu, &popup_key, &popup_mod);

   if (ZLIST (widget)->focus >= 0)
      thumb = zalbum_get_cell_data (ZALBUM (widget), ZLIST (widget)->focus);

   if (keyval == popup_key && (!popup_mod || (modval & popup_mod))) {
      thumbview_popup_menu (tv, NULL, NULL);
   } else {
      switch (keyval) {
      case GDK_Left:
      case GDK_Right:
      case GDK_Up:
      case GDK_Down:
         return TRUE;
      case GDK_Return:
         if (!thumb) break;
         if (modval & GDK_SHIFT_MASK || modval & GDK_CONTROL_MASK) {
            /* is there somteing to do? */
         } else {
            thumbview_set_selection_all (tv, FALSE);
         }
         thumbview_set_selection (thumb, TRUE);
         thumbview_open_image (tv, thumb, 0);
         break;
      case GDK_space:
         if (!thumb) break;
         thumbview_set_selection (thumb, !thumb->selected);
         break;
      case GDK_Delete:
         thumbview_delete_files (tv);
         break;
      default:
         break;
      }
   }

   return FALSE;
}


/******************************************************************************
 *
 *   private functions.
 *
 ******************************************************************************/
static ThumbViewData *
thumbtable2_new (ThumbView *tv)
{
   ThumbViewData *tv_data;

   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   if (!tv_data) {
      tv_data = g_new0 (ThumbViewData, 1);
      tv_data->album = NULL;
      g_hash_table_insert (tv->disp_mode_data, THUMBTABLE2_LABEL, tv_data);
   }

   return tv_data;
}



static gboolean
thumbtable2_append_thumb_frame (ThumbView *tv, Thumbnail *thumb, gchar *dest_mode)
{
   ThumbViewData *tv_data;
   const gchar *filename;
   gchar *label;
   GdkPixmap *pixmap = NULL;
   GdkBitmap *mask = NULL;
   guint idx;

   g_return_val_if_fail (tv, FALSE);
   g_return_val_if_fail (thumb, FALSE);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_val_if_fail (tv_data, FALSE);

   filename = g_basename(image_info_get_path (thumb->info));

   if (!strcmp (dest_mode, LIST2_THUMB_LABEL)) {
      label = listview_create_label_str (thumb);
   } else {
      label = charset_to_internal (filename,
                                   conf.charset_filename,
                                   conf.charset_auto_detect_fn,
                                   conf.charset_filename_mode);
   }

   idx = zalbum_add (ZALBUM (tv_data->album), label);
   g_free (label);

   zalbum_set_cell_data (ZALBUM (tv_data->album), idx, (gpointer) thumb);

   thumbtable2_set_selection (thumb, thumb->selected);

   if (!strcmp (LIST2_ICON_LABEL, dest_mode)) {
      thumbnail_get_icon (thumb, &pixmap, &mask);
   } else {
      thumbnail_get_thumb (thumb, &pixmap, &mask);
   }
   if (!pixmap) return FALSE;

   zalbum_set_pixmap (ZALBUM (tv_data->album), idx, pixmap, mask);

   return TRUE;
}


/******************************************************************************
 *
 *   public functions.
 *
 ******************************************************************************/
void
thumbtable2_remove_thumbview_data (ThumbView *tv)
{
   ThumbViewData *tv_data;

   if (!tv) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   if (!tv_data) return;

   g_hash_table_remove (tv->disp_mode_data, THUMBTABLE2_LABEL); 
   g_free (tv_data);
}


GList *
thumbtable2_append_thumb_frames (ThumbView *tv,
                                 GList *start,
                                 gchar *dest_mode)
{
   GList *node, *loadlist = NULL;
   gboolean thumb_loaded;

   g_return_val_if_fail (tv, FALSE);

   node = start;
   while (node) {
      Thumbnail  *thumb = node->data;

      thumb_loaded = thumbtable2_append_thumb_frame (tv, thumb, dest_mode);
      if (!thumb_loaded)
         loadlist = g_list_append (loadlist, thumb);

      node = g_list_next (node);
   }

   return loadlist;
}


GtkWidget *
thumbtable2_add_thumbnail (Thumbnail *thumb,
                           gchar *dest_mode,
                           ThumbLoadType type)
{
   ThumbView *tv;
   ThumbViewData *tv_data;
   GdkPixmap *pixmap = NULL;
   GdkBitmap *mask = NULL;
   GList *node;
   gint pos;

   g_return_val_if_fail (thumb, NULL);

   tv = thumb->thumb_view;
   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_val_if_fail (tv_data && tv_data->album, NULL);

   node = g_list_find (tv->thumblist, thumb);
   pos = g_list_position (tv->thumblist, node);

   /* set thumbnail */
   if (!strcmp (LIST2_ICON_LABEL, dest_mode)) {
      thumbnail_get_icon (thumb, &pixmap, &mask);
   } else {
      thumbnail_get_thumb (thumb, &pixmap, &mask);
   }
   if (pixmap)
      zalbum_set_pixmap (ZALBUM (tv_data->album), pos, pixmap, mask);

   return NULL;
}


void
thumbtable2_redraw (ThumbView *tv, gchar *dest_mode,
                    GtkWidget *scroll_win, GList **loadlist)
{
   ThumbViewData *tv_data;
   GList *node;

   g_return_if_fail (tv);

   node = g_list_find (ThumbViewList, tv);
   if (!node) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   if (!tv_data)
      tv_data = thumbtable2_new (tv);

   if (scroll_win) {
      if (GTK_BIN (tv->container)->child)
         gtk_widget_destroy (GTK_BIN (tv->container)->child);   
      tv_data->album = thumbtable2_create (tv, dest_mode);
      gtk_container_add (GTK_CONTAINER (scroll_win), tv_data->album);
   }

   if (!loadlist) return;

   /* create thumbnail load list */
   *loadlist = NULL;
   node = tv->thumblist;
   while (node) {
      Thumbnail *thumb = node->data;
      GdkPixmap *pixmap = NULL;
      GdkBitmap *mask = NULL;

      if (!strcmp (LIST2_ICON_LABEL, dest_mode)) {
         thumbnail_get_icon (thumb, &pixmap, &mask);
      } else {
         thumbnail_get_thumb (thumb, &pixmap, &mask);
      }
      if (!pixmap)
         *loadlist = g_list_append (*loadlist, thumb);

      node = g_list_next (node);
   }
}


GtkWidget *
thumbtable2_resize (ThumbView *tv)
{
   ThumbViewData *tv_data;
   GList *node;

   g_return_val_if_fail (tv, NULL);

   node = g_list_find (ThumbViewList, tv);
   if (!node) return NULL;

   thumbtable2_redraw (tv, tv->disp_mode, tv->container, NULL);;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_val_if_fail (tv_data, NULL);

   return tv_data->album;
}


gboolean
thumbtable2_refresh_thumbnail (Thumbnail *thumb, ThumbLoadType type)
{
   GtkWidget *widget;
   ThumbView *tv;

   if (!thumb) return FALSE;

   tv = thumb->thumb_view;
   if (!tv) return FALSE;

   widget = thumbtable2_add_thumbnail (thumb, tv->disp_mode, type);

   if (widget)
      return TRUE;
   else
      return FALSE;
}


gboolean
thumbtable2_set_selection (Thumbnail *thumb, gboolean select)
{
   ThumbView *tv;
   ThumbViewData *tv_data;
   GList *node;
   gint pos;

   g_return_val_if_fail (thumb, FALSE);

   tv = thumb->thumb_view;
   g_return_val_if_fail (tv, FALSE);
   if (g_list_length (tv->thumblist) < 1) return FALSE;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_val_if_fail (tv_data && tv_data->album, FALSE);

   node = g_list_find (tv->thumblist, thumb);
   pos = g_list_position (tv->thumblist, node);

   if (pos >= 0) {
      thumb->selected = select;
      if (thumb->selected)
         zlist_cell_select (ZLIST (tv_data->album), pos);
      else
         zlist_cell_unselect (ZLIST (tv_data->album), pos);
   }

   return TRUE;
}


gboolean
thumbtable2_thumbnail_is_in_viewport (ThumbView *tv, Thumbnail *thumb)
{
   ThumbViewData *tv_data;
   GList *node;
   gint index;
   gboolean success;
   GdkRectangle area, cell_area, intersect_area;

   g_return_val_if_fail (tv, FALSE);
   g_return_val_if_fail (thumb, FALSE);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_val_if_fail (tv_data, FALSE);

   node = g_list_find (tv->thumblist, thumb);
   index = g_list_position (tv->thumblist, node);

   /* widget area */
   gtkutil_get_widget_area (tv_data->album, &area);

   /* cell area */
   success = zlist_get_cell_area (ZLIST (tv_data->album), index, &cell_area);
   g_return_val_if_fail (success, FALSE);

   /* intersect? */
   if (gdk_rectangle_intersect (&area, &cell_area, &intersect_area))
      return TRUE;
   else
      return FALSE;
}


void
thumbtable2_adjust (ThumbView *tv, Thumbnail *thumb)
{
   ThumbViewData *tv_data;
   GList *node;
   gint pos;

   g_return_if_fail (tv);

   node = g_list_find (ThumbViewList, tv);
   if (!node) return;

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   g_return_if_fail (tv_data);

   node = g_list_find (tv->thumblist, thumb);
   if (!node) return;
   pos = g_list_position (tv->thumblist, node);

   zlist_moveto (ZLIST (tv_data->album), pos);

   return;
}


GtkWidget *
thumbtable2_create (ThumbView *tv, gchar *dest_mode)
{
   ThumbViewData *tv_data;

   g_return_val_if_fail (tv, NULL);

   tv_data = g_hash_table_lookup (tv->disp_mode_data, THUMBTABLE2_LABEL);
   if (!tv_data) {
      tv_data = thumbtable2_new (tv);
      g_return_val_if_fail (tv_data, NULL);
   }

   /* create zalbum widget */
   tv_data->album = zalbum_new ();

   if (!strcmp (LIST2_ICON_LABEL, dest_mode)) {
      gtk_widget_set_name (tv_data->album, "List2IconMode");
      zlist_set_to_horizontal (ZLIST (tv_data->album));
      zalbum_set_label_position (ZALBUM (tv_data->album),
                                 ZALBUM_CELL_LABEL_RIGHT);
      zalbum_set_min_pixmap_size (ZALBUM (tv_data->album),
                                  ICON_SIZE, ICON_SIZE);
   } else if (!strcmp (LIST2_THUMB_LABEL, dest_mode)) {
      gtk_widget_set_name (tv_data->album, "List2ThumbMode");
      listview_create_title_idx_list  ();
      zlist_set_to_horizontal (ZLIST (tv_data->album));
      zalbum_set_label_position (ZALBUM (tv_data->album),
                                 ZALBUM_CELL_LABEL_RIGHT);
      zalbum_set_min_pixmap_size (ZALBUM (tv_data->album),
                                  tv->ThumbnailSize,
                                  tv->ThumbnailSize);
   } else {
      gtk_widget_set_name (tv_data->album, "Thumbnail2Mode");
      zalbum_set_min_pixmap_size (ZALBUM (tv_data->album),
                                  tv->ThumbnailSize,
                                  tv->ThumbnailSize);
   }

   scrolled_set_auto_scroll (SCROLLED (tv_data->album),
                             SCROLLED_AUTO_SCROLL_BOTH
                             | SCROLLED_AUTO_SCROLL_DND
                             | SCROLLED_AUTO_SCROLL_MOTION);

   scrolled_set_h_auto_scroll_resolution (SCROLLED (tv_data->album), 20, 10);
   scrolled_set_v_auto_scroll_resolution (SCROLLED (tv_data->album), 20, 10);

   zlist_set_selection_mode (ZLIST (tv_data->album), GTK_SELECTION_EXTENDED);

   if (!strcmp (THUMBTABLE2_LABEL, dest_mode)) {
      zlist_set_cell_padding (ZLIST (tv_data->album),
                              conf.thumbtable_col_space,
                              conf.thumbtable_row_space);
   } else {
      zlist_set_cell_padding (ZLIST (tv_data->album), 5, 5);
   }

   gtk_widget_show (tv_data->album);

   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "button_press_event",
                             GTK_SIGNAL_FUNC (cb_album_button_press), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "button_release_event",
                             GTK_SIGNAL_FUNC (cb_album_button_release), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "key-press-event",
                             GTK_SIGNAL_FUNC (cb_album_key_press), tv);

   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "cell_select",
                             GTK_SIGNAL_FUNC (cb_select_cell), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "cell_unselect",
                             GTK_SIGNAL_FUNC (cb_unselect_cell), tv);

   /* Drag and Drop */
   dnd_src_set  (tv_data->album, dnd_types_uri, dnd_types_uri_num);
   dnd_dest_set (tv_data->album, dnd_types_uri, dnd_types_uri_num);
   gtk_object_set_data (GTK_OBJECT (tv_data->album), "gimv-tab", tv);

   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "drag_begin",
                             GTK_SIGNAL_FUNC (thumbview_drag_begin_cb), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "drag_data_get",
                             GTK_SIGNAL_FUNC (thumbview_drag_data_get_cb), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "drag_data_received",
                             GTK_SIGNAL_FUNC (thumbview_drag_data_received_cb), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "drag-data-delete",
                             GTK_SIGNAL_FUNC (thumbview_drag_data_delete_cb), tv);
   gtk_signal_connect_after (GTK_OBJECT (tv_data->album), "drag_end",
                             GTK_SIGNAL_FUNC (thumbview_drag_end_cb), tv);

   /* append thumbnail frames */
   if (tv->thumblist) {
      thumbtable2_append_thumb_frames (tv, tv->thumblist, dest_mode);
      zlist_cell_set_focus (ZLIST (tv_data->album), 0);
   }

   return tv_data->album;
}
