/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: plugin.h,v 1.16 2002/10/17 22:50:04 makeinu Exp $
 */

#ifndef __PLUGIN_H__
#define __PLUGIN_H__

#include <gmodule.h>

typedef enum
{
   PLUGIN_IMAGE_LOADER,
   PLUGIN_IMAGE_SAVER,
   PLUGIN_IMAGE_EFFECTER,
   PLGUIN_ARCHIVER,
   PLUGIN_EXT_ARCHIVER,       /* Wrapper for external command */
   PLUGIN_MOVIE,              /* maybe never use (use xine instead) */
   PLUGIN_THUMBVIEW_EMBED,    /* Thumbnail View dislpay mode */
   PLUGIN_THUMB_CACHE,
   PLUGIN_DATABASE,
   PLUGIN_G_IO,               /* Stream handling */
   PLUGIN_MISC
} PluginType;

typedef enum {
   PLUGIN_PREFS_STRING,
   PLUGIN_PREFS_INT,
   PLUGIN_PREFS_FLOAT,
   PLUGIN_PREFS_BOOL,
} PluginPrefsType;


typedef GtkWidget *(*PluginAboutFn)  (gpointer data);
typedef GtkWidget *(*PluginConfigFn) (gpointer data);


typedef struct PluginInfo_Tag
{
   const PluginType    type;
   const gchar * const name;
   const gchar * const version;
   const gchar * const author;
   const gchar * const description;
   PluginAboutFn       about;         /* open about window  */
   PluginConfigFn      config;        /* open config window */
   const gint          n_entries;     /* count of "implement" entry */
   gpointer            implement;
} PluginInfo;


/* plugin list */
void   plugin_create_search_dir_list   (void);
GList *plugin_get_search_dir_list      (void);

/* get each infomation from plugin */
const gchar *plugin_get_name           (GModule *module);
const gchar *plugin_get_version_string (GModule *module);
const gchar *plugin_get_author         (GModule *module);
const gchar *plugin_get_module_name    (GModule *module);
GList       *plugin_get_list           (PluginType type);

/* plugin preference */
gboolean plugin_prefs_save_value       (const gchar     *pname,
                                        PluginType       ptype,
                                        const gchar     *key,
                                        const gchar     *value);
gboolean plugin_prefs_load_value       (const gchar     *name,
                                        PluginType       ptype,
                                        const gchar     *key,
                                        PluginPrefsType  type,
                                        gpointer         data);
void     plugin_prefs_read_files       (void);
void     plugin_prefs_write_files      (void);

void     plugin_init                   (void);

#endif /* __PLUGIN_H__ */
