/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001-2003 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: prefs_ui_mplayer.c,v 1.1.2.3 2003/01/30 19:13:31 makeinu Exp $
 */

#include "prefs_ui_mplayer.h"

#ifdef ENABLE_MPLAYER

#include "gtkutils.h"
#include "gimv_mplayer.h"


extern Config   *config_changed;
extern Config   *config_prechanged;


static void
cb_vo_combo_changed (GtkEditable *editable, gpointer data)
{
   const gchar *text = gtk_entry_get_text (GTK_ENTRY (editable));

   if (config_changed->mplayer_vo_driver != config_prechanged->mplayer_vo_driver)
      g_free (config_changed->mplayer_vo_driver);
   config_changed->mplayer_vo_driver = NULL;

   if (text && *text)
      config_changed->mplayer_vo_driver = g_strdup (text);
}


static void
cb_ao_combo_changed (GtkEditable *editable, gpointer data)
{
   const gchar *text = gtk_entry_get_text (GTK_ENTRY (editable));

   if (config_changed->mplayer_ao_driver != config_prechanged->mplayer_ao_driver)
      g_free (config_changed->mplayer_ao_driver);
   config_changed->mplayer_ao_driver = NULL;

   if (text && *text)
      config_changed->mplayer_ao_driver = g_strdup (text);
}


GtkWidget *
prefs_mplayer_page (void)
{
   GtkWidget *main_vbox, *frame, *vbox, *hbox;
   GtkWidget *label, *vo_combo, *ao_combo, *mplayer, *spinner, *toggle;
   GtkAdjustment *adj;
   const GList *list;

   main_vbox = gtk_vbox_new (FALSE, 0);

   /**********************************************
    * Driver Frame
    **********************************************/
   prefs_ui_create_frame(_("Driver"), frame, vbox, main_vbox, FALSE);


   /* video driver combo */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER (hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);

   label = gtk_label_new (_("Video driver : "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
   gtk_widget_show (label);

   vo_combo = gtk_combo_new ();
   gtk_box_pack_start (GTK_BOX (hbox), vo_combo, FALSE, FALSE, 2);
   gtk_widget_show (vo_combo);
   gtk_widget_set_usize (vo_combo, 100, -1);

   /* audio driver combo */
   label = gtk_label_new (_("Audio driver : "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
   gtk_widget_show (label);

   ao_combo = gtk_combo_new ();
   gtk_box_pack_start (GTK_BOX (hbox), ao_combo, FALSE, FALSE, 2);
   gtk_widget_show (ao_combo);
   gtk_widget_set_usize (ao_combo, 100, -1);


   /**********************************************
    * Thumbnail Frame
    **********************************************/
   prefs_ui_create_frame(_("Thumbnail"), frame, vbox, main_vbox, FALSE);

   /* Ignore filename extension or not */
   toggle = gtkutil_create_check_button (_("Enable creating thumbnail of movie using MPlayer"),
                                         conf.mplayer_thumbnail_enable,
                                         gtkutil_get_data_from_toggle_cb,
                                         &config_changed->mplayer_thumbnail_enable);
   gtk_container_set_border_width (GTK_CONTAINER(toggle), 5);
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
   gtk_widget_show (toggle);

   hbox = gtk_hbox_new (FALSE, 5);
   gtk_container_set_border_width (GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   label = gtk_label_new (_("Stream position : "));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
   adj = (GtkAdjustment *) gtk_adjustment_new (conf.mplayer_thumbnail_pos,
                                               0.0, 100.0, 0.01, 0.1, 0.0);
   spinner = gtkutil_create_spin_button (adj);
   gtk_widget_set_usize(spinner, 70, -1);
   gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spinner), 2);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (gtkutil_get_data_from_adjustment_by_float_cb),
                       &config_changed->mplayer_thumbnail_pos);
   gtk_box_pack_start (GTK_BOX (hbox), spinner, FALSE, FALSE, 0);
   gtk_widget_show (spinner);

   label = gtk_label_new (_("[%]"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);



   /* for detecting drivers */
   mplayer = gimv_mplayer_new ();
#ifdef USE_GTK2
   g_object_ref (G_OBJECT (mplayer));
   gtk_object_sink (GTK_OBJECT (mplayer));
#endif /* USE_GTK2 */

   /* set drivers list */
   list = gimv_mplayer_get_video_out_drivers (GIMV_MPLAYER (mplayer), FALSE);
   gtk_combo_set_popdown_strings (GTK_COMBO (vo_combo), (GList *) list);
   list = gimv_mplayer_get_audio_out_drivers (GIMV_MPLAYER (mplayer), FALSE);
   gtk_combo_set_popdown_strings (GTK_COMBO (ao_combo), (GList *) list);

   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(vo_combo)->entry),
                       conf.mplayer_vo_driver);
   gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(ao_combo)->entry),
                       conf.mplayer_ao_driver);

   gtk_signal_connect (GTK_OBJECT (GTK_COMBO(vo_combo)->entry), "changed",
                       GTK_SIGNAL_FUNC (cb_vo_combo_changed), NULL);
   gtk_signal_connect (GTK_OBJECT (GTK_COMBO(ao_combo)->entry), "changed",
                       GTK_SIGNAL_FUNC (cb_ao_combo_changed), NULL);

   gtk_widget_unref (mplayer);

   return main_vbox;
}

#endif /* ENABLE_MPLAYER */
