/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * dstack_list.c - pair of reorderable stack list widget.
 * Copyright (C) 2001-2002 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 3S30, Boston, MA 02111-1307, USA.
 *
 * $Id: dstack_list.h,v 1.3 2002/11/01 20:01:40 makeinu Exp $
 */

#ifndef __DSTACK_LIST_H__
#define __DSTACK_LIST_H__

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <gtk/gtk.h>

#define DSTACK_LIST(obj)         GTK_CHECK_CAST (obj, dstack_list_get_type (), DStackList)
#define DSTACK_LIST_CLASS(klass) GTK_CHECK_CLASS_CAST (klass, dstack_list_get_type, DStackListClass)
#define IS_DSTACK_LIST(obj)      GTK_CHECK_TYPE (obj, dstack_list_get_type ())


typedef struct DStackList_Tag      DStackList;
typedef struct DStackListClass_Tag DStackListClass;


struct DStackList_Tag {
   GtkHBox     parent;

   /* public (read only) */
   GtkWidget   *clist1;
   GtkWidget   *clist2;
   GtkWidget   *add_button;
   GtkWidget   *del_button;
   GtkWidget   *up_button;
   GtkWidget   *down_button;

   /* private */
   gint         clist1_rows;
   gint         clist1_selected;
   gint         clist1_dest_row;
   gint         clist2_rows;
   gint         clist2_selected;
   gint         clist2_dest_row;
   GList       *available_list;
};


struct DStackListClass_Tag {
   GtkHBoxClass parent_class;

#if 0
   gboolean (*available_list_updated) (DStackList *dslist); /* need?*/
#endif
   gboolean (*enabled_list_updated)   (DStackList *dslist);
};


/*
 *  "available" or "clist1" means left side CList (or TreeView).
 *  "enabled" or "clist2" means right side CList (or TreeView).
 */

guint         dstack_list_get_type              (void);

GtkWidget    *dstack_list_new                   (const gchar  *clist1_title,
                                                 const gchar  *clist2_title);

/* FIXME: item should be untranslated text.
   Currently, DStackList translate item text by itself. */
gint          dstack_list_append_available_item (DStackList   *dslist,
                                                 const gchar  *item);
void          dstack_list_column_add            (DStackList  *dslist,
                                                 gint         idx);
void          dstack_list_column_del            (DStackList  *dslist,
                                                 gint         idx);
void          dstack_list_column_add_by_label   (DStackList   *dslist,
                                                 const gchar  *label);
gint          dstack_list_get_n_available_items (DStackList  *dslist);
gint          dstack_list_get_n_enabled_items   (DStackList  *dslist);

/* FIXME: currently, this function returns the untranslated text */
gchar        *dstack_list_get_enabled_row_text  (DStackList   *dslist,
                                                 gint          row);


#if 0   /* API draft */

void          dstack_list_up                    (DStackList  *dslist,
                                                 gint         idx);
void          dstack_list_down                  (DStackList  *dslist,
                                                 gint         idx);
gboolean      dstack_list_swap_idx              (DStackList  *dslist,
                                                 gint         src_idx,
                                                 gint         dest_idx);

void          dstack_list_clist1_set_reorderble (DStackList  *dslist,
                                                 gboolean     reorderble);

void          dstack_list_get_idx_by_label      (DStackList  *dslist,
                                                 const gchar *label);
void          dstack_list_get_idx_by_data       (DStackList  *dslist,
                                                 gpointer    *data);
gboolean      dstack_list_idx_is_enabled        (DStackList  *dslist,
                                                 gint         idx);
const gchar  *dstack_list_get_label             (DStackList  *dslist,
                                                 gint         idx);
gpointer      dstack_list_get_data              (DStackList  *dslist,
                                                 gint         idx);

/* returned list should be free, but its content data shouldn't be free. */
GList        *dstack_list_get_available_list    (DStackList *dslist);
GList        *dstack_list_get_enabled_list      (DStackList *dslist);

void          dstack_list_set_translate_func   (DStackList   *dstack_list,
                                                nanndakke?);
#endif

#endif /* __DSTACK_LIST_H__ */
