/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: scrolled.h,v 1.3 2002/10/03 12:27:37 makeinu Exp $
 */

/*
 *  These codes are mostly taken from Another X image viewer.
 *
 *  Another X image viewer Author:
 *     David Ramboz <dramboz@users.sourceforge.net>
 */

#ifndef _SCROLLED_H_
#define _SCROLLED_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>


#define SCROLLED_TYPE            scrolled_get_type()
#define SCROLLED(obj)            GTK_CHECK_CAST(obj, SCROLLED_TYPE, Scrolled)
#define SCROLLED_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), SCROLLED_TYPE, ScrolledClass))
#define IS_SCROLLED(obj)         GTK_CHECK_TYPE (obj, SCROLLED_TYPE)
#define SCROLLED_X(scrolled, x)  (-SCROLLED(scrolled)->x_offset + (x))
#define SCROLLED_Y(scrolled, y)  (-SCROLLED(scrolled)->y_offset + (y))
#define SCROLLED_VX(scrolled, x) (SCROLLED(scrolled)->x_offset + (x))
#define SCROLLED_VY(scrolled, y) (SCROLLED(scrolled)->y_offset + (y))


typedef struct _Scrolled Scrolled;
typedef struct _ScrolledClass ScrolledClass;

typedef enum
{
   SCROLLED_AUTO_SCROLL_NONE       = 0,
   SCROLLED_AUTO_SCROLL_HORIZONTAL = 1 << 0,
   SCROLLED_AUTO_SCROLL_VERTICAL   = 1 << 1,
   SCROLLED_AUTO_SCROLL_BOTH       = 1 << 2,

   SCROLLED_AUTO_SCROLL_DND        = 1 << 3, /* for drag motion event */
   SCROLLED_AUTO_SCROLL_MOTION     = 1 << 4, /* for motion notify event */
   SCROLLED_AUTO_SCROLL_MOTION_ALL = 1 << 5, /* do not check whether button
                                                was pressed or not */
} ScrolledAutoScrollFlags;

struct _Scrolled {
   GtkContainer   container;

   gint           x_offset;
   gint           y_offset;

   GtkAdjustment *h_adjustment;
   GtkAdjustment *v_adjustment;

   GdkGC         *copy_gc;
   guint          freeze_count;

   /* for auto scroll */
   gint          autoscroll_flags;
   gint          scroll_edge_x, scroll_edge_y;
   gint          x_step, y_step;
   gint          x_interval, y_interval;
   gboolean      pressed;
   gint          drag_start_vx;    /* virtual x of drag start point */
   gint          drag_start_vy;    /* virtual y of drag start point */
   gint          drag_motion_x;    /* current real x in the window */
   gint          drag_motion_y;    /* current real y in the window */
   gfloat        step_scale;
   gint          hscroll_timer_id;
   gint          vscroll_timer_id;
};


struct _ScrolledClass {
   GtkContainerClass parent_class;

   void              (*set_scroll_adjustments) (GtkWidget *widget, 
                                                GtkAdjustment *hadjustment, 
                                                GtkAdjustment *vadjustment);
   void              (*adjust_adjustments)     (Scrolled *scrolled);
};


GtkType    scrolled_get_type (void);

void       scrolled_realize                      (Scrolled *scrolled);
void       scrolled_unrealize                    (Scrolled *scrolled);
void       scrolled_freeze                       (Scrolled *scrolled);
void       scrolled_thawn                        (Scrolled *scrolled);
void       scrolled_page_up                      (Scrolled *scrolled);
void       scrolled_page_down                    (Scrolled *scrolled);
void       scrolled_page_left                    (Scrolled *scrolled);
void       scrolled_page_right                   (Scrolled *scrolled);
void       scrolled_set_auto_scroll              (Scrolled *scrolled,
                                                  ScrolledAutoScrollFlags flags);
void       scrolled_unset_auto_scroll            (Scrolled *scrolled);
void       scrolled_set_auto_scroll_edge_width   (Scrolled *scrolled,
                                                  gint      x_edge,
                                                  gint      y_edge); /* -1 to use default value */
void       scrolled_set_h_auto_scroll_resolution (Scrolled *scrolled,
                                                  gint      step,
                                                  gint      interval);
void       scrolled_set_v_auto_scroll_resolution (Scrolled *scrolled,
                                                  gint      step,
                                                  gint      interval);
gboolean   scrolled_is_dragging                  (Scrolled *scrolled);
void       scrolled_stop_auto_scroll             (Scrolled *scrolled);

#endif /* _SCROLLED_H_ */
