/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: zalbum.h,v 1.2 2002/10/02 19:05:10 makeinu Exp $
 */

/*
 *  These codes are mostly taken from ProView image viewer.
 *
 *  ProView image viewer Author:
 *     promax <promax@users.sourceforge.net>
 */

/*
 *  modification file from Another X image viewer
 *  David Ramboz <dramboz@users.sourceforge.net>
 */

#ifndef __ZALBUM_H__
#define __ZALBUM_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "zlist.h"

#define ZALBUM_TYPE            (zalbum_get_type ())
#define ZALBUM(widget)         (GTK_CHECK_CAST ((widget), ZALBUM_TYPE, ZAlbum))
#define ZALBUM_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), ZALBUM_TYPE, ZAlbumClass))
#define IS_ZALBUM(widget)      (GTK_CHECK_TYPE ((widget), ZALBUM_TYPE))
#define IS_ZALBUM_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), ZALBUM_TYPE))

typedef struct _ZAlbum            ZAlbum;
typedef struct _ZAlbumCell        ZAlbumCell;
typedef struct _ZAlbumClass       ZAlbumClass;
typedef struct _ZAlbumColumnInfo  ZAlbumColumnInfo;

enum
{
   ZALBUM_CELL_SELECTED = 1 << 1,
};

typedef enum
{
   ZALBUM_CELL_LABEL_BOTTOM,
   ZALBUM_CELL_LABEL_TOP,
   ZALBUM_CELL_LABEL_RIGHT,
   ZALBUM_CELL_LABEL_LEFT,
} ZAlbumLabelPosition;

struct _ZAlbum
{
   ZList list;

   guint8 label_pos;

   gint max_pix_width;
   gint max_pix_height;

   gint max_cell_width;
   gint max_cell_height;

   gint len;
};

struct _ZAlbumClass
{
   ZListClass parent_class;
};

struct _ZAlbumCell
{
   gint flags;
   const gchar *name;

   GdkPixmap *ipix;
   GdkBitmap *imask;

   gpointer user_data;
   GtkDestroyNotify destroy;
};


GtkType    zalbum_get_type                  (void);
GtkWidget *zalbum_new                       (void);
guint      zalbum_add                       (ZAlbum           *album,
                                             const gchar      *name);
void       zalbum_set_label_position        (ZAlbum           *album,
                                             ZAlbumLabelPosition pos);
void       zalbum_set_min_pixmap_size       (ZAlbum           *album,
                                             guint             width,
                                             guint             height);
void       zalbum_set_min_cell_size         (ZAlbum           *album,
                                             guint             width,
                                             guint             height);
void       zalbum_set_pixmap                (ZAlbum           *album,
                                             guint             idx,
                                             GdkPixmap        *pixmap,
                                             GdkBitmap        *mask);
void       zalbum_set_cell_data             (ZAlbum           *album,
                                             guint             idx,
                                             gpointer          user_data);
void       zalbum_set_cell_data_full        (ZAlbum           *album,
                                             guint             idx,
                                             gpointer          data,
                                             GtkDestroyNotify  destroy);
gpointer   zalbum_get_cell_data             (ZAlbum           *album,
                                             guint             idx);

#define zalbum_freeze(album) (scrolled_freeze(SCROLLED(album)))
#define zalbum_thawn(album)  (scrolled_thawn(SCROLLED(album)))

#endif /* __ZALBUM_H__ */
