/* -*- Mode: C; tab-width: 3; indent-tabs-mode: nil; c-basic-offset: 3 -*- */

/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: zlist.h,v 1.4 2002/10/03 12:27:37 makeinu Exp $
 */

/*
 *  These codes are mostly taken from Another X image viewer.
 *
 *  Another X image viewer Author:
 *     David Ramboz <dramboz@users.sourceforge.net>
 */


#ifndef _ZLIST_H_
#define _ZLIST_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "scrolled.h"

#define ZLIST_TYPE            (zlist_get_type ())
#define ZLIST(widget)         (GTK_CHECK_CAST ((widget), ZLIST_TYPE, ZList))
#define ZLIST_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), ZLIST_TYPE, ZListClass))
#define IS_ZLIST(widget)      (GTK_CHECK_TYPE ((widget), ZLIST_TYPE))
#define IS_ZLIST_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), ZLIST_TYPE))

enum {
   ZLIST_HORIZONTAL        = 1 << 1,
   ZLIST_1                 = 1 << 2,
   ZLIST_USES_DND          = 1 << 3,   /* not implemented yet */
   ZLIST_HIGHLIGHTED       = 1 << 4
};

typedef enum {
   ZLIST_REGION_SELECT_OFF,
   ZLIST_REGION_SELECT_NORMAL,
   ZLIST_REGION_SELECT_EXPAND,    /* shift mask */
   ZLIST_REGION_SELECT_TOGGLE     /* control mask */
} ZListRegionSelectMode;

typedef struct _ZList ZList;
typedef struct _ZListClass ZListClass;


struct _ZList
{
   Scrolled               scrolled;

   guint                  flags;

   gint                   cell_width;
   gint                   cell_height;

   gint                   rows;
   gint                   columns;

   GArray                *cells;
   gint                   cell_count;

   gint                   selection_mode;
   GList                 *selection;
   gint                   focus;
   gint                   anchor;

   ZListRegionSelectMode  region_select;
   GdkGC                 *region_line_gc;
   GList                 *selection_mask;

   gint                   cell_x_pad, cell_y_pad;
   gint                   x_pad, y_pad;

   GtkWidget             *entered_cell;
};

struct _ZListClass
{
   ScrolledClass parent_class;

   void    (*clear)             (ZList          *list);
   void    (*cell_draw)         (ZList          *list,
                                 gpointer        cell,
                                 GdkRectangle   *cell_area,
                                 GdkRectangle   *area);
   void    (*cell_size_request) (ZList          *list,
                                 gpointer        cell,
                                 GtkRequisition *requisition);
   void    (*cell_draw_focus)   (ZList          *list,
                                 gpointer        cell,
                                 GdkRectangle   *cell_area);
   void    (*cell_draw_default) (ZList          *list,
                                 gpointer        cell,
                                 GdkRectangle   *cell_area);
   void    (*cell_select)       (ZList          *list,
                                 gint            index);
   void    (*cell_unselect)     (ZList          *list,
                                 gint            index);
};


GtkType    zlist_get_type                    (void);

void       zlist_construct                   (ZList            *list,
                                              gint              flags);
GtkWidget *zlist_new                         (guint             flags);
void       zlist_set_to_vertical             (ZList            *zlist);
void       zlist_set_to_horizontal           (ZList            *zlist);
guint      zlist_add                         (ZList            *list,
                                              gpointer          cell);
void       zlist_remove                      (ZList            *list,
                                              gpointer          cell);
void       zlist_clear                       (ZList            *list);
void       zlist_set_selection_mode          (ZList            *list,
                                              GtkSelectionMode  mode);
void       zlist_set_cell_padding            (ZList            *list,
                                              gint              x_pad,
                                              gint              y_pad);
void       zlist_set_cell_size               (ZList            *list,
                                              gint              width,
                                              gint              height);
void       zlist_set_1                       (ZList            *list,
                                              gint              one);
gpointer   zlist_cell_from_xy                (ZList            *list,
                                              gint              x,
                                              gint              y);
gint       zlist_cell_index_from_xy          (ZList            *list,
                                              gint              x,
                                              gint              y);
void       zlist_cell_select                 (ZList            *list,
                                              gint              index);
void       zlist_cell_unselect               (ZList            *list,
                                              gint              index);
void       zlist_cell_toggle                 (ZList            *list,
                                              gint              index);
void       zlist_unselect_all                (ZList            *list);
void       zlist_extend_selection            (ZList            *list,
                                              gint              to);
void       zlist_cell_select_by_region       (ZList            *list,
                                              guint             start_col,
                                              guint             start_row,
                                              guint             end_col,
                                              guint             end_row);
void       zlist_cell_select_by_pixel_region (ZList            *list,
                                              gint              start_x,
                                              gint              start_y,
                                              gint              end_x,
                                              gint              end_y);
void       zlist_set_selection_mask          (ZList            *list,
                                              GList            *mask_list);
void       zlist_unset_selection_mask        (ZList            *list);
void       zlist_moveto                      (ZList            *list,
                                              gint              index);
void       zlist_cell_set_focus              (ZList            *list,
                                              gint              index);
void       zlist_cell_unset_focus            (ZList            *list);

gboolean   zlist_get_cell_area               (ZList *list,
                                              gint index,
                                              GdkRectangle *area);

/* protected */
#define    ZLIST_CELL_FROM_INDEX(list, index) \
   (g_array_index (ZLIST(list)->cells, gpointer, index))

void       zlist_draw_cell          (ZList    *list,
                                     gint      index);
gint       zlist_cell_index         (ZList    *list,
                                     gpointer  cell);
gint       zlist_update_cell_size   (ZList    *list,
                                     gpointer  cell);

#endif /* _ZLIST_H_ */
